@@ -1,6 +1,6 @@
 #if !defined(lint) && !defined(SABER)
 static char sccsid[] = "@(#)db_dump.c	4.33 (Berkeley) 3/3/91";
-static char rcsid[] = "$Id: db_dump.c,v 8.8 1995/12/06 20:34:38 vixie Exp $";
+static char rcsid[] = "$Id: db_dump.c,v 8.14 1996/08/05 08:31:30 vixie Exp $";
 #endif /* not lint */
 
 /*
@@ -58,8 +58,10 @@ static char rcsid[] = "$Id: db_dump.c,v 8.8 1995/12/06 20:34:38 vixie Exp $";
  * --Copyright--
  */
 
+#include <sys/types.h>
 #include <sys/param.h>
 #include <sys/stat.h>
+#include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/nameser.h>
 #include <arpa/inet.h>
@@ -168,7 +170,7 @@ scan_root(htp)
 	soon.tv_sec += TIMBUF;
 
 	for (np = htp->h_tab[0]; np != NULL; np = np->n_next) {
-		if (np->n_dname[0] == '\0') {
+		if (NAME(*np)[0] == '\0') {
 			dp = np->n_data;
 			while (dp != NULL) {
 				if (dp->d_type == T_NS &&
@@ -314,13 +316,14 @@ zt_dump(fp)
 			  : "Nil",
 			zp->z_type, zp->z_class,
 			zp->z_source ? zp->z_source : "Nil");
-		fprintf(fp, ";\ttime=%ld, lastupdate=%ld, serial=%u,\n",
-			zp->z_time, zp->z_lastupdate, zp->z_serial);
+		fprintf(fp, ";\ttime=%lu, lastupdate=%lu, serial=%u,\n",
+			(u_long)zp->z_time, (u_long)zp->z_lastupdate,
+			zp->z_serial);
 		fprintf(fp, ";\trefresh=%u, retry=%u, expire=%u, minimum=%u\n",
 			zp->z_refresh, zp->z_retry,
 			zp->z_expire, zp->z_minimum);
-		fprintf(fp, ";\tftime=%ld, xaddr=[%s], state=%04x, pid=%d\n",
-			zp->z_ftime, inet_ntoa(zp->z_xaddr),
+		fprintf(fp, ";\tftime=%lu, xaddr=[%s], state=%04x, pid=%d\n",
+			(u_long)zp->z_ftime, inet_ntoa(zp->z_xaddr),
 			zp->z_flags, (int)zp->z_xferpid);
 		sprintf(buf, ";\tz_addr[%d]: ", zp->z_addrcnt);
 		pre = buf;
@@ -397,14 +400,14 @@ db_dump(htp, fp, zone, origin)
 			}
 #endif /*NCACHE*/
 			if (found_data == 0 || found_data == 2) {
-			    if (np->n_dname[0] == 0) {
-				if (origin[0] == 0)
+			    if (NAME(*np)[0] == '\0') {
+				if (origin[0] == '\0')
 				    fprintf(fp, ".\t");
 				else
 				    fprintf(fp, ".%s.\t", origin); /* ??? */
 			    } else
-				fprintf(fp, "%s\t", np->n_dname);
-			    if (strlen(np->n_dname) < (size_t)8)
+				fprintf(fp, "%s\t", NAME(*np));
+			    if (NAMELEN(*np) < (size_t)8)
 				tab = 1;
 			    found_data++;
 			} else {
@@ -419,7 +422,7 @@ db_dump(htp, fp, zone, origin)
 			    else
 				    fprintf(fp, "%d\t",
 				        (int)(dp->d_ttl - tt.tv_sec));
-			} else if (dp->d_ttl != 0 &&
+			} else if (dp->d_ttl != USE_MINIMUM &&
 			    dp->d_ttl != zones[dp->d_zone].z_minimum)
 				fprintf(fp, "%d\t", (int)dp->d_ttl);
 			else if (tab)
@@ -559,10 +562,23 @@ db_dump(htp, fp, zone, origin)
 							    dp->d_data, NULL),
 					     fp);
 				break;
-#ifdef LOC_RR
-			case T_LOC:
-				(void) fputs(loc_ntoa(dp->d_data, NULL), fp);
+			case T_AAAA: {
+				char t[sizeof
+				"ffff:ffff:ffff:ffff:ffff:ffff:255.255.255.255"
+				      ];
+
+				(void) fputs(inet_ntop(AF_INET6, dp->d_data,
+						       t, sizeof t),
+					     fp);
 				break;
+			}
+#ifdef LOC_RR
+			case T_LOC: {
+				char t[256];
+
+				(void) fputs(loc_ntoa(dp->d_data, t), fp);
+				break;
+			}
 #endif /* LOC_RR */
 			case T_UINFO:
 				fprintf(fp, "\"%s\"", cp);
@@ -584,7 +600,7 @@ db_dump(htp, fp, zone, origin)
 				fputs(inet_ntoa(*(struct in_addr *)&addr), fp);
 				proto = protocolname(*cp);
 				cp += sizeof(char); 
-				fprintf(fp, "%s ", proto);
+				fprintf(fp, " %s ", proto);
 				i = 0;
 				while(cp < (u_char *)dp->d_data + dp->d_size) {
 					j = *cp++;
@@ -809,7 +825,7 @@ atob(inbuf, inbuflen, outbuf, outbuflen, numbytes)
 	}
 
 	/* Get byte count and checksum information from end of buffer */
-	if(sscanf(inp, "%ld %lx %lx %lx", numbytes, &oeor, &osum, &orot) != 4)
+	if (sscanf(inp, "%d %lx %lx %lx", numbytes, &oeor, &osum, &orot) != 4)
 		return(CONV_BADFMT);
 	if ((oeor != Ceor) || (osum != Csum) || (orot != Crot))
 		return(CONV_BADCKSUM);
@@ -899,7 +915,7 @@ btoa(inbuf, inbuflen, outbuf, outbuflen)
 	}
 	/* Put byte count and checksum information at end of buffer, delimited
 	   by 'x' */
-	(void) sprintf(outp, "x %ld %lx %lx %lx", inbuflen, Ceor, Csum, Crot);
+	(void) sprintf(outp, "x %d %lx %lx %lx", inbuflen, Ceor, Csum, Crot);
 	if (&outp[strlen(outp) - 1] >= endoutp)
 		return(CONV_OVERFLOW);
 	else
