@@ -1,6 +1,6 @@
 #if !defined(lint) && !defined(SABER)
 static char sccsid[] = "@(#)db_update.c	4.28 (Berkeley) 3/21/91";
-static char rcsid[] = "$Id: db_update.c,v 8.7 1995/12/06 20:34:38 vixie Exp $";
+static char rcsid[] = "$Id: db_update.c,v 8.12 1996/08/05 08:31:30 vixie Exp $";
 #endif /* not lint */
 
 /*
@@ -61,6 +61,7 @@ static char rcsid[] = "$Id: db_update.c,v 8.7 1995/12/06 20:34:38 vixie Exp $";
 #include <stdio.h>
 #include <syslog.h>
 
+#include <sys/types.h>
 #include <sys/param.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -111,7 +112,7 @@ isRefByNS(name, htp)
 
 
 /* int
- * findMyZone(struct namebuf *np)
+ * findMyZone(struct namebuf *np, int class)
  *	surf the zone cuts and find this zone the hard way
  * return value:
  *	zone number or DB_Z_CACHE if it's outside a zone
@@ -132,7 +133,7 @@ findMyZone(np, class)
 	struct namebuf *np;
 	register int class;
 {
-	for (; np; np = np->n_parent) {
+	for (; np; np = np_parent(np)) {
 		register struct databuf *dp;
 
 		/* if we encounter an SOA, we're in its zone (which can be
@@ -159,7 +160,8 @@ findMyZone(np, class)
 
 
 #ifdef NO_GLUE
-#define ISVALIDGLUE(xdp) ((xdp)->d_type == T_NS || (xdp)->d_type == T_A)
+#define ISVALIDGLUE(xdp) ((xdp)->d_type == T_NS || (xdp)->d_type == T_A \
+			  || (xdp)->d_type == T_AAAA)
 #else
 #define ISVALIDGLUE(xdp) (1)
 #endif /*NO_GLUE*/
@@ -535,9 +537,10 @@ db_update(name, odp, newdp, flags, htp)
 					if (odp->d_ttl > dp->d_ttl)
 						dp->d_ttl = odp->d_ttl;
 					dprintf(3, (ddt,
-						"db_update: new ttl %ld +%d\n",
+					       "db_update: new ttl %ld +%ld\n",
 						    (u_long)dp->d_ttl,
-						    dp->d_ttl - tt.tv_sec));
+						    (u_long)
+						     (dp->d_ttl - tt.tv_sec)));
 				}
 				return (DATAEXISTS);
 			}
@@ -653,6 +656,7 @@ db_cmp(dp1, dp2)
 	case T_WKS:
 	case T_NULL:
 	case T_NSAP:
+	case T_AAAA:
 	case T_LOC:
 #ifdef ALLOW_T_UNSPEC
         case T_UNSPEC:
