@@ -1,6 +1,6 @@
 #if !defined(lint) && !defined(SABER)
 static char sccsid[] = "@(#)ns_init.c	4.38 (Berkeley) 3/21/91";
-static char rcsid[] = "$Id: ns_init.c,v 8.12 1995/12/29 07:16:18 vixie Exp $";
+static char rcsid[] = "$Id: ns_init.c,v 8.17 1996/08/05 08:31:30 vixie Exp $";
 #endif /* not lint */
 
 /*
@@ -58,6 +58,7 @@ static char rcsid[] = "$Id: ns_init.c,v 8.12 1995/12/29 07:16:18 vixie Exp $";
  * --Copyright--
  */
 
+#include <sys/types.h>
 #include <sys/param.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
@@ -84,12 +85,16 @@ static void		zoneinit __P((struct zoneinfo *)),
 #endif
 			free_forwarders __P((void)),
 			ns_limit __P((const char *name, int value)),
+			ns_checknames __P((const char *names,
+					   const char *severity)),
 			ns_rlimit __P((const char *name, enum limit limit,
 				       long value)),
 			ns_option __P((const char *name));
 
 static struct zoneinfo	*find_zone __P((char *, int, int));
 
+static enum severity	checkname_severity[num_trans];
+
 /*
  * Set new refresh time for zone.  Use a random number in the last half of
  * the refresh limit; we want it to be substantially correct while still
@@ -130,6 +135,11 @@ ns_init(bootfile)
 	dprintf(1, (ddt, "\nns_init(%s)\n", bootfile));
 	gettime(&tt);
 
+	memset(checkname_severity, '\0', sizeof checkname_severity);
+	checkname_severity[primary_trans] = fail;
+	checkname_severity[secondary_trans] = warn;
+	checkname_severity[response_trans] = ignore;
+
         if (loads == 0) {
 		if ((zones =
 			(struct zoneinfo *)calloc(64, sizeof(struct zoneinfo)))
@@ -277,6 +287,11 @@ boot_read(filename, includefile)
 			while (getword(buf, sizeof(buf), fp, 0))
 				ns_option(buf);
 			continue;
+		} else if (strcasecmp(buf, "check-names") == 0) {
+			(void) getword(buf, sizeof(buf), fp, 0);
+			(void) getword(obuf, sizeof(obuf), fp, 0);
+			ns_checknames(buf, obuf);
+			continue;
 		} else if (strcasecmp(buf, "forwarders") == 0) {
 			get_forwarders(fp);
 			continue;
@@ -756,7 +771,7 @@ get_forwarders(fp)
 
 	dprintf(1, (ddt, "forwarders "));
 
-	/* on mulitple forwarder lines, move to end of the list */
+	/* On multiple forwarder lines, move to end of the list. */
 #ifdef SLAVE_FORWARD
 	if (fwdtab != NULL){
 		forward_count++;
@@ -899,6 +914,140 @@ ns_limit(name, value)
 	}
 }
 
+static int
+select_string(strings, string)
+	const char *strings[];
+	const char *string;
+{
+	int i;
+
+	for (i = 0; strings[i] != NULL; i++)
+		if (!strcasecmp(strings[i], string))
+			return (i);
+	return (-1);
+}
+
+static void
+ns_checknames(transport_str, severity_str)
+	const char *transport_str;
+	const char *severity_str;
+{
+	enum transport transport;
+	enum severity severity;
+	int i;
+
+	if ((i = select_string(transport_strings, transport_str)) == -1) {
+		syslog(LOG_ERR,
+		      "error: unrecognized transport type in bootfile: \"%s\"",
+		       transport_str);
+		exit(1);
+	}
+	transport = (enum transport) i;
+
+	if ((i = select_string(severity_strings, severity_str)) == -1) {
+		syslog(LOG_ERR,
+		       "error: unrecognized severity type in bootfile: \"%s\"",
+		       severity_str);
+		exit(1);
+	}
+	severity = (enum severity) i;
+
+	checkname_severity[transport] = severity;
+	syslog(LOG_INFO, "check-names %s %s", transport_str, severity_str);
+}
+
+enum context
+ns_ptrcontext(owner)
+	const char *owner;
+{
+	if (samedomain(owner, "in-addr.arpa") || samedomain(owner, "ip6.int"))
+		return (hostname_ctx);
+	return (domain_ctx);
+}
+
+enum context
+ns_ownercontext(type, transport)
+	int type;
+	enum transport transport;
+{
+	enum context context;
+
+	switch (type) {
+	case T_A:
+	case T_WKS:
+	case T_MX:
+		switch (transport) {
+		case primary_trans:
+		case secondary_trans:
+			context = owner_ctx;
+			break;
+		case response_trans:
+			context = hostname_ctx;
+			break;
+		default:
+			abort();
+		}
+		break;
+	case T_MB:
+	case T_MG:
+		context = mailname_ctx;
+	default:
+		context = domain_ctx;
+		break;
+	}
+	return (context);
+}
+
+int
+ns_nameok(name, class, transport, context)
+	const char *name;
+	int class;
+	enum transport transport;
+	enum context context;
+{
+	int ok = 1;
+	enum severity severity = checkname_severity[transport];
+
+	if (severity == ignore)
+		return (1);
+	switch (context) {
+	case domain_ctx:
+		ok = (class != C_IN) || res_dnok(name);
+		break;
+	case owner_ctx:
+		ok = (class != C_IN) || res_ownok(name);
+		break;
+	case mailname_ctx:
+		ok = res_mailok(name);
+		break;
+	case hostname_ctx:
+		ok = res_hnok(name);
+		break;
+	default:
+		abort();
+	}
+	if (!ok) {
+		syslog((transport == response_trans) ? LOG_INFO : LOG_NOTICE,
+		       "%s name \"%s %s\" (%s) is invalid - %s",
+		       context_strings[context],
+		       name, p_class(class),
+		       transport_strings[transport],
+		       (severity == fail) ? "rejecting" : "proceeding anyway");
+		if (severity == warn)
+			ok = 1;
+	}
+	return (ok);
+}
+
+int
+ns_wildcard(name)
+	const char *name;
+{
+	if (*name != '*')
+		return (0);
+	return (*++name == '\0');
+}
+
 static void
 ns_rlimit(name, limit, value)
 	const char *name;
@@ -926,7 +1075,7 @@ ns_rlimit(name, limit, value)
 		syslog(LOG_WARNING, "getrlimit(%s): %m", name);
 		return;
 	}
-	limits.rlim_cur = value;
+	limits.rlim_cur = limits.rlim_max = value;
 	if (setrlimit(rlimit, &limits) < 0) {
 		syslog(LOG_WARNING, "setrlimit(%s, %ld): %m", name, value);
 		return;
