@@ -1,6 +1,6 @@
 #if !defined(lint) && !defined(SABER)
 static char sccsid[] = "@(#)ns_main.c	4.55 (Berkeley) 7/1/91";
-static char rcsid[] = "$Id: ns_main.c,v 8.13 1996/01/09 20:23:55 vixie Exp $";
+static char rcsid[] = "$Id: ns_main.c,v 8.17 1996/08/05 08:31:30 vixie Exp $";
 #endif /* not lint */
 
 /*
@@ -70,6 +70,7 @@ char copyright[] =
  * Internet Name server (see RCF1035 & others).
  */
 
+#include <sys/types.h>
 #include <sys/param.h>
 #include <sys/file.h>
 #include <sys/stat.h>
@@ -314,7 +315,7 @@ main(argc, argv, envp)
 #ifdef WANT_PIDFILE
 	/* tuck my process id away */
 #ifdef PID_FIX
-	fp = fopen(PidFile, "r+");
+	fp = fopen(PidFile, "w");
 	if (fp != NULL) {
 		(void) fgets(oldpid, sizeof(oldpid), fp);
 		(void) rewind(fp);
@@ -896,7 +897,9 @@ main(argc, argv, envp)
 			 * Consult database to get the answer.
 			 */
 			if (sp->s_size == 0) {
+#ifdef XSTATS
 				nameserIncr(sp->s_from.sin_addr, nssRcvdTCP);
+#endif
 				sq_query(sp);
 				ns_req(sp->s_buf,
 				       sp->s_bufp - sp->s_buf,
@@ -1048,7 +1051,7 @@ getnetconf()
 				netloop.mask = 0xffffffff;
 				netloop.addr = ntp->my_addr.s_addr;
 				dprintf(1, (ddt, "loopback address: x%lx\n",
-					    netloop.my_addr.s_addr));
+					    (u_long)netloop.my_addr.s_addr));
 			}
 			continue;
 		} else if ((ifreq.ifr_flags & IFF_POINTOPOINT)) {
