@@ -1,6 +1,6 @@
 #if !defined(lint) && !defined(SABER)
 static char sccsid[] = "@(#)ns_maint.c	4.39 (Berkeley) 3/2/91";
-static char rcsid[] = "$Id: ns_maint.c,v 8.11 1995/12/22 10:20:30 vixie Exp $";
+static char rcsid[] = "$Id: ns_maint.c,v 8.16 1996/08/05 08:31:30 vixie Exp $";
 #endif /* not lint */
 
 /*
@@ -165,7 +165,7 @@ ns_maint()
 	}
 #ifdef CLEANCACHE
 	if ((cache_time + cache_interval) <= tt.tv_sec) {
-		if (cache_time)
+		if (cache_time && (!NoRecurse || !NoFetchGlue))
 			remove_zone(hashtab, 0, 0);
 		cache_time = tt.tv_sec;
 	}
@@ -331,9 +331,9 @@ qserial_answer(qp, serial)
 		if (!haveComplained((char*)zp, "went backward")) {
 			syslog(LOG_NOTICE,
    "Zone \"%s\" (class %d) SOA serial# (%lu) rcvd from [%s] is < ours (%lu)\n",
-			       zp->z_origin, zp->z_class, serial,
+			       zp->z_origin, zp->z_class, (u_long)serial,
 			       inet_ntoa(from_addr.sin_addr),
-			       zp->z_serial);
+			       (u_long)zp->z_serial);
 		}
 	} else {
 		dprintf(1, (ddt, "qserial_answer: zone serial is still OK\n"));
@@ -479,12 +479,11 @@ startxfer(zp)
 	argv[argc] = 0;
 
 #ifdef DEBUG
-#ifdef ECHOARGS
 	if (debug) {
 		for (i = 0; i < argc; i++) 
-			fprintf(ddt, "Arg %d=%s\n", i, argv[i]);
+			fprintf(ddt, " %s", argv[i]);
+		fprintf(ddt, "\n");
         }
-#endif /* ECHOARGS */
 #endif /* DEBUG */
 
 	gettime(&tt);
@@ -572,7 +571,7 @@ printzoneinfo(zonenum)
 	if (zp->z_time) {
 		fprintf(ddt, ", now time : %lu sec", (u_long)tt.tv_sec);
 		fprintf(ddt, ", time left: %lu sec",
-			(long)(zp->z_time - tt.tv_sec));
+			(u_long)(zp->z_time - tt.tv_sec));
 	}
 	fprintf(ddt, "; flags %lx\n", (u_long)zp->z_flags);
 }
@@ -657,7 +656,7 @@ purge_zone(dname, htp, class)
 
 	dprintf(1, (ddt, "purge_zone(%s,%d)\n", dname, class));
 	if ((np = nlookup(dname, &phtp, &fname, 0)) && dname == fname &&
-	    !WILDCARD_P(dname)) {
+	    !ns_wildcard(NAME(*np))) {
 		for (pdp = NULL, dp = np->n_data; dp != NULL; ) {
 			if (dp->d_class == class)
 				dp = rm_datum(dp, np, pdp);
