@@ -1,6 +1,6 @@
 #if !defined(lint) && !defined(SABER)
 static char sccsid[] = "@(#)ns_req.c	4.47 (Berkeley) 7/1/91";
-static char rcsid[] = "$Id: ns_req.c,v 8.15 1995/12/29 07:16:18 vixie Exp $";
+static char rcsid[] = "$Id: ns_req.c,v 8.20 1996/08/05 08:31:30 vixie Exp $";
 #endif /* not lint */
 
 /*
@@ -58,6 +58,7 @@ static char rcsid[] = "$Id: ns_req.c,v 8.15 1995/12/29 07:16:18 vixie Exp $";
  * --Copyright--
  */
 
+#include <sys/types.h>
 #include <sys/param.h>
 #include <sys/uio.h>
 #include <sys/file.h>
@@ -448,14 +449,10 @@ req_query(hp, cpp, eom, qsp, buflenp, msglenp, msg, dfd, from)
 #endif
 	struct databuf *dp;
 
-	nameserIncr(from->sin_addr, nssRcvdQ);
-	
 #ifdef XSTATS
-	/* Statistics for queries coming from port <> 53, suspect some kind of forwarder */
-	if (from->sin_port != ns_port)
-		nameserIncr(from->sin_addr, nssNotNsQ);
+	nameserIncr(from->sin_addr, nssRcvdQ);
 #endif
-
+	
 #ifdef DATUMREFCNT
 	nsp[0] = NULL;
 #endif
@@ -738,7 +735,6 @@ try_again:
 #endif /*BIND_NOTIFY*/
 	if (type == T_AXFR) {
 		startxfr(qsp, np, msg, *cpp - msg, class, dname);
-		sqrm(qsp);
 		return (Return);
 	}
 
@@ -916,7 +912,7 @@ fetchns:
 		** for the root servers.
 		*/
 		if (np) {
-			if (np->n_dname[0] == '\0') {
+			if (NAME(*np)[0] == '\0') {
 				syslog(LOG_NOTICE, 
 				       "ns_req: no address for root server");
 				hp->rcode = SERVFAIL;
@@ -954,7 +950,7 @@ fetchns:
 					goto do_servfail;
 				break;
 			}
-			np = np->n_parent;
+			np = np_parent(np);
 		}
 		goto fetchns;	/* Try again. */
 	case FW_SERVFAIL:
@@ -982,7 +978,9 @@ req_iquery(hp, cpp, eom, buflenp, msg, from)
 	int dlen, alen, n, type, class, count;
 	char dnbuf[MAXDNAME], anbuf[PACKETSZ], *data, *fname;
 
+#ifdef XSTATS
 	nameserIncr(from->sin_addr, nssRcvdIQ);
+#endif
 
 	if (ntohs(hp->ancount) != 1
 	    || ntohs(hp->qdcount) != 0
@@ -1067,7 +1065,7 @@ req_iquery(hp, cpp, eom, buflenp, msg, from)
 
 			if ((np = ip->i_dname[i]) == NULL)
 				break;
-			dprintf(5, (ddt, "dname = %d\n", np->n_dname));
+			dprintf(5, (ddt, "dname = %d\n", NAME(*np)));
 			for (dp = np->n_data; dp != NULL; dp = dp->d_next) {
 				if (!match(dp, class, type))
 					continue;
@@ -1230,7 +1228,7 @@ make_rr(name, dp, buf, buflen, doadd)
 
 	dprintf(5, (ddt, "make_rr(%s, %lx, %lx, %d, %d) %d zone %d ttl %lu\n",
 		    name, (u_long)dp, (u_long)buf,
-		    buflen, doadd, dp->d_size, dp->d_zone, dp->d_ttl));
+		    buflen, doadd, dp->d_size, dp->d_zone, (u_long)dp->d_ttl));
 
 #ifdef	NCACHE
 	if (dp->d_rcode
@@ -1244,13 +1242,11 @@ make_rr(name, dp, buf, buflen, doadd)
 	zp = &zones[dp->d_zone];
 	/* check for outdated RR before updating dnptrs by dn_comp() (?) */
 	if (zp->z_type == Z_CACHE) {
-		ttl = dp->d_ttl - (u_int32_t) tt.tv_sec;
-		if ((dp->d_flags & DB_F_HINT) || (ttl < 0)) {
-			dprintf(3, (ddt,
-				    "make_rr: %d=>0, %#lx\n",
-				    ttl, (u_long)dp->d_flags));
+		if ((dp->d_flags & DB_F_HINT) != 0
+		    || dp->d_ttl < (u_int32_t)tt.tv_sec) {
 			ttl = 0;
-		}
+		} else
+			ttl = dp->d_ttl - (u_int32_t) tt.tv_sec;
 	} else {
 		if (dp->d_ttl != USE_MINIMUM)
 			ttl = dp->d_ttl;
@@ -1488,7 +1484,9 @@ doaddinfo(hp, msg, msglen)
 				break;
 			}
 			if (!match(dp, (int)ap->a_class, T_A) &&
-			    !match(dp, C_IN, T_A)) {
+			    !match(dp, C_IN, T_A) &&
+			    !match(dp, (int)ap->a_class, T_AAAA) &&
+			    !match(dp, C_IN, T_AAAA)) {
 				continue;
 			}
 			foundany++;
@@ -1496,7 +1494,7 @@ doaddinfo(hp, msg, msglen)
 				foundstale++;
 				dprintf(1, (ddt,
 					    "doaddinfo: stale entry '%s'%s\n",
-					    np->n_dname,
+					    NAME(*np),
 					    (dp->d_flags&DB_F_HINT)
 					        ? " hint"
 					        : ""
@@ -1546,7 +1544,7 @@ doaddinfo(hp, msg, msglen)
 		if (foundcname) {
 			if (!haveComplained((char*)nhash(ap->a_dname),
 					    (char*)nhash(ap->a_rname))) {
-				syslog(LOG_INFO,
+				syslog(LOG_DEBUG,
 				       "\"%s %s %s\" points to a CNAME (%s)",
 				       ap->a_rname, p_class(ap->a_class),
 				       p_type(ap->a_rtype), ap->a_dname);
@@ -1652,7 +1650,7 @@ doaxfr(np, rfp, top, class)
 			continue;
 		fwritemsg(rfp, msg, n + HFIXEDSZ);
 #ifdef NO_GLUE
-		if ((np != top) || (top->n_dname[0] == '\0')) {
+		if ((np != top) || (NAME(*top)[0] == '\0')) {
 #endif /*NO_GLUE*/
 		    /*  Glue the sub domains together by sending 
 		     *  the address records for the sub domain
@@ -1671,7 +1669,7 @@ doaxfr(np, rfp, top, class)
 		    for (tnp = gnp; tnp != NULL; tnp = tnp->n_parent)
 			if ( tnp == top )
 			    break;
-		    if ( (tnp == NULL) && (top->n_dname[0] != '\0') )
+		    if ( (tnp == NULL) && (NAME(*top)[0] != '\0') )
 			continue;  /* name server is not below top domain */
 		    for (tnp = gnp;
 			 tnp != NULL && tnp != top;
@@ -1690,7 +1688,7 @@ doaxfr(np, rfp, top, class)
 			    break; /* found a zone cut */
 		    }
 		    if ((tnp == top) ||
-			    ((tnp == NULL) && (top->n_dname[0] == '\0')))
+			    ((tnp == NULL) && (NAME(*top)[0] == '\0')))
 			continue;  /* name server is not in a delegated zone */
 		    /* now we know glue records are needed.  send them. */
 #endif /*NO_GLUE*/
@@ -1784,7 +1782,7 @@ doaxfr(np, rfp, top, class)
 	nppend = npp + htp->h_size;
 	while (npp < nppend) {
 		for (np = *npp++; np != NULL; np = np->n_next) {
-		    if (np->n_dname[0] != '\0') { /* don't redo root domain */
+		    if (NAME(*np)[0] != '\0') { /* don't redo root domain */
 			doaxfr(np, rfp, top, class);
 		    }
 		}
@@ -2009,8 +2007,8 @@ printSOAdata(dp)
 	fprintf(ddt, "printSOAdata: in-charge(%#lx)='%s'\n",
 		(u_long)cp, cp);
 	cp += strlen(cp) + 1; /* skip in-charge string */
-	fprintf(ddt, "printSOAdata: serial(%lx)=%d\n",
-		cp, (u_long)_getlong(cp));
+	fprintf(ddt, "printSOAdata: serial(%lx)=%lu\n",
+		(u_long)cp, (u_long)_getlong(cp));
 }
 #endif
 #endif
@@ -2027,6 +2025,8 @@ startxfr(qsp, np, soa, soalen, class, dname)
 	FILE *rfp;
 	int fdstat;
 	pid_t pid;
+	int pipefd[2];
+	char c;
 #ifdef HAVE_SETVBUF
 	char *buf;
 #endif
@@ -2039,6 +2039,14 @@ startxfr(qsp, np, soa, soalen, class, dname)
 
 	dprintf(5, (ddt, "startxfr()\n"));
 
+	/* create a pipe to synchronize parent and child */
+	if (pipe(pipefd) != 0) {
+		syslog(LOG_NOTICE, "startxfr(%s -> %s) failing; pipe: %m",
+		       dname, sin_ntoa(&qsp->s_from));
+		sqrm(qsp);
+		return;
+	}
+
 	/*
 	 * child does the work while
 	 * the parent continues
@@ -2047,6 +2055,9 @@ startxfr(qsp, np, soa, soalen, class, dname)
 	case -1:
 		syslog(LOG_NOTICE, "startxfr(%s -> %s) failing; fork: %m",
 		       dname, sin_ntoa(&qsp->s_from));
+		close(pipefd[0]);
+		close(pipefd[1]);
+		sqrm(qsp);
 		return;
 	case 0:
 		/* child */
@@ -2054,7 +2065,11 @@ startxfr(qsp, np, soa, soalen, class, dname)
 	default:
 		/* parent */
 		syslog(LOG_DEBUG, "zone transfer of \"%s\" to %s (pid %lu)",
-		       dname, sin_ntoa(&qsp->s_from), pid);
+		       dname, sin_ntoa(&qsp->s_from), (u_long)pid);
+		close(pipefd[0]);	/* close the read end */
+		sqrm(qsp);
+		/* close the write end to release the child */
+		close(pipefd[1]);
 		return;
 	}
 
@@ -2068,6 +2083,19 @@ startxfr(qsp, np, soa, soalen, class, dname)
 	sqflush(/*allbut*/ qsp);
 	dqflush((time_t)0);
 
+	close(pipefd[1]);		/* close the write end */
+	/*
+	 * Wait for parent to close the write end of the pipe which
+	 * we'll see as an EOF.  The parent won't close the write end
+	 * until it has closed the fd we'll be writing to, at which
+	 * point it will be safe for us to proceed.
+         *
+	 * We shouldn't get interrupted, but ...
+	 */
+	while (read(pipefd[0], &c, 1) == -1 && errno == EINTR)
+		; /* nothing */
+	close(pipefd[0]);
+
 #ifdef RENICE
 	nice(-40);  nice(20);  nice(0);		/* back to "normal" */
 #endif
