@@ -1,6 +1,6 @@
 #if !defined(lint) && !defined(SABER)
 static char sccsid[] = "@(#)ns_resp.c	4.65 (Berkeley) 3/3/91";
-static char rcsid[] = "$Id: ns_resp.c,v 8.19 1996/01/09 20:23:55 vixie Exp $";
+static char rcsid[] = "$Id: ns_resp.c,v 8.27 1996/08/05 08:31:30 vixie Exp $";
 #endif /* not lint */
 
 /*
@@ -58,6 +58,7 @@ static char rcsid[] = "$Id: ns_resp.c,v 8.19 1996/01/09 20:23:55 vixie Exp $";
  * --Copyright--
  */
 
+#include <sys/types.h>
 #include <sys/param.h>
 #include <sys/socket.h>
 #include <sys/file.h>
@@ -91,7 +92,7 @@ static const char	skipnameFailedAnswer[] = "skipname failed in answer",
 			dlenUnderrunAnswer[] =	"dlen underrun in answer",
 			outofDataFinal[] =	"out of data in final pass",
 			outofDataAFinal[] =	"out of data after final pass",
-			editFailed[] =		"edit of response failed";
+			badNameFound[] =	"found an invalid domain name";
 
 static char *
 learntFrom(qp, server)
@@ -194,7 +195,7 @@ ns_resp(msg, msglen)
 	int count, founddata, foundname;
 	int buflen;
 	int newmsglen;
-	char name[MAXDNAME], qname[MAXDNAME];
+	char name[MAXDNAME], qname[MAXDNAME], msgbuf[MAXDNAME*2];
 	char *dname;
 	const char *fname;
 	const char *formerrmsg = "brain damage";
@@ -246,6 +247,11 @@ ns_resp(msg, msglen)
 		cp += n;
 		GETSHORT(qtype, cp);
 		GETSHORT(qclass, cp);
+		if (!ns_nameok(qname, qclass, response_trans,
+			       ns_ownercontext(qtype, response_trans))) {
+			formerrmsg = badNameFound;
+			goto formerr;
+		}
 		if (cp > eom) {
 			formerrmsg = outofDataQuery;
 			goto formerr;
@@ -253,8 +259,6 @@ ns_resp(msg, msglen)
 		if (qp->q_msg && qp->q_msglen &&
 		    !res_nameinquery(qname, qtype, qclass,
 				     qp->q_msg, qp->q_msg + qp->q_msglen)) {
-			char msgbuf[MAXDNAME*2];
-
 			sprintf(msgbuf,
 				"query section mismatch (%s %s %s)",
 				qname, p_class(qclass), p_type(qtype));
@@ -540,6 +544,11 @@ ns_resp(msg, msglen)
 			formerrmsg = outofDataAuth;
 			goto formerr;
 		}
+		if (!ns_nameok(name, class, response_trans,
+			       ns_ownercontext(type, response_trans))) {
+			formerrmsg = badNameFound;
+			goto formerr;
+		}
 
 		/*
 		 * If the answer delegates us either to the same level in
@@ -603,11 +612,14 @@ ns_resp(msg, msglen)
 				formerrmsg = outofDataAnswer;
 				goto formerr;
 			}
+			if (!ns_nameok(name, class, response_trans,
+				       ns_ownercontext(type, response_trans))){
+				formerrmsg = badNameFound;
+				goto formerr;
+			}
 			if (strcasecmp(qname, name) ||
 			    qtype != type ||
 			    qclass != class) {
-				char msgbuf[MAXDNAME*2];
-
 				sprintf(msgbuf,
 					"qserial answer mismatch (%s %s %s)",
 					name, p_class(class), p_type(type));
@@ -632,8 +644,10 @@ ns_resp(msg, msglen)
 			GETLONG(serial, tp);
 
 			qserial_answer(qp, serial);
+			qremove(qp);
+		} else {
+			retry(qp);
 		}
-		qremove(qp);
 		return;
 	}
 
@@ -683,6 +697,18 @@ ns_resp(msg, msglen)
 			count -= ancount;	/* things are pretty grim */
 		}
 		/* XXX - should retry this query with TCP */
+		/*
+		 * XXX - if this response is forwarded to the client
+		 * the truncated section is included.  We will not
+		 * validate it, and if it somehow corrupt, we won't
+		 * notice.
+		 *
+		 * XXX - if the answer section is truncated and we got
+		 * this response after being redirected by a CNAME, we
+		 * will not include any part of the final answer in our
+		 * response to the client.  This will make the client
+		 * think that there are no RRs of the appropriate type.
+		 */
 	}
 
 	tp = cp;
@@ -716,6 +742,11 @@ ns_resp(msg, msglen)
 		tempcp += n;
 		GETSHORT(type, tempcp);
 		GETSHORT(class, tempcp);
+		if (!ns_nameok(name, class, response_trans,
+			       ns_ownercontext(type, response_trans))) {
+			formerrmsg = badNameFound;
+			goto formerr;
+		}
 
 		/*
 		 * See if there are any NS RRs in the authority section
@@ -874,11 +905,21 @@ ns_resp(msg, msglen)
 		return;
 	}
 
-	if (ancount && !validanswer)
+	if (ancount && count && !validanswer)
 		/*
 		 * Everything passed validation but we didn't get the
 		 * final answer.  The response must have contained
 		 * a dangling CNAME.  Force a restart of the query.
+		 *
+		 * Don't set restart if count==0, since this means
+		 * the response was truncated in the answer section,
+	         * causing us to set count to 0 which will cause
+		 * validanswer to be 0 as well even though the answer
+		 * section probably contained valid RRs (just not
+		 * a complete set).
+		 * XXX - this works right if we can just forward this
+		 * response to the client, but not if we found a CNAME
+		 * in a prior response and restarted the query.
 		 */
 		restart = 1;
 
@@ -973,6 +1014,10 @@ ns_resp(msg, msglen)
 		dprintf(1, (ddt, "dn_expand failed\n"));
 		goto servfail;
 	}
+	if (!res_dnok(dname)) {
+		dprintf(1, (ddt, "bad name (%s)\n", dname));
+		goto servfail;
+	}
 	cp += n + QFIXEDSZ;
 	buflen = sizeof(newmsg) - (cp - newmsg);
 
@@ -1111,12 +1156,28 @@ ns_resp(msg, msglen)
 	if ((n = nslookup(nsp, qp, dname, "ns_resp")) <= 0) {
 		if (n < 0) {
 			dprintf(3, (ddt, "resp: nslookup reports danger\n"));
+			if (cname)  /* a remote CNAME that does not have data */
+				goto return_newmsg;
+			goto servfail;
 		} else {
 			dprintf(3, (ddt, "resp: no addrs found for NS's\n"));
+			/*
+			 * Timeout while sysquery looks up the NS addresses.
+			 *
+			 * Hopefully we'll have them when the client asks
+			 * again.
+			 *
+			 * too bad we can't just wait for the sysquery
+			 * response to restart this query (it's too hard).
+			 *
+			 * We could try to crawl back up the tree looking
+			 * for reachable servers, but we may have just
+			 * gotten delegated down here by a response with
+			 * no A RRs for the servers.  If we blindly tried
+			 * this strategy, we bang on the same server forever.
+			 */
+			goto timeout;
 		}
-		if (cname)	/* a remote CNAME that does not have data */
-			goto return_newmsg;
-		goto servfail;
 	}
 	for (n = 0; (u_int)n < qp->q_naddr; n++)
 		qp->q_addr[n].stime.tv_sec = 0;
@@ -1177,7 +1238,9 @@ ns_resp(msg, msglen)
 		nameserIncr(nsa->sin_addr, nssSendtoErr);
 	}
 	hp->rd = 0;	/* leave set to 0 for dup detection */
+#ifdef XSTATS
 	nameserIncr(nsa->sin_addr, nssSentFwdR);
+#endif
 	nameserIncr(qp->q_from.sin_addr, nssRcvdFwdR);
 	dprintf(3, (ddt, "resp: Query sent.\n"));
 #ifdef	DATUMREFCNT
@@ -1190,7 +1253,9 @@ ns_resp(msg, msglen)
 			    (char*)nhash(formerrmsg)))
 		syslog(LOG_INFO, "Malformed response from %s (%s)\n",
 		       sin_ntoa(&from_addr), formerrmsg);
+#ifdef XSTATS
 	nameserIncr(from_addr.sin_addr, nssSentFErr);
+#endif
 #ifdef	DATUMREFCNT
 	free_nsp(nsp);
 #endif
@@ -1198,7 +1263,9 @@ ns_resp(msg, msglen)
 
  return_msg:
 	nameserIncr(from_addr.sin_addr, nssRcvdFwdR);
+#ifdef XSTATS
 	nameserIncr(qp->q_from.sin_addr, nssSentFwdR);
+#endif
 	/* The "standard" return code */
 	hp->qr = 1;
 	hp->id = qp->q_id;
@@ -1235,15 +1302,18 @@ ns_resp(msg, msglen)
 	return;
 
  servfail:
+#ifdef XSTATS
 	nameserIncr(qp->q_from.sin_addr, nssSentFail);
-	hp = (HEADER *)(cname ? qp->q_cmsg : qp->q_msg);
+#endif
+	hp = (HEADER *)(qp->q_cmsglen ? qp->q_cmsg : qp->q_msg);
 	hp->rcode = SERVFAIL;
 	hp->qr = 1;
 	hp->id = qp->q_id;
 	hp->rd = 1;
 	hp->ra = (NoRecurse == 0);
-	(void) send_msg((u_char *)hp, (cname ? qp->q_cmsglen : qp->q_msglen),
+	(void) send_msg((u_char *)hp, (qp->q_cmsglen ? qp->q_cmsglen : qp->q_msglen),
 			qp);
+ timeout:
 	qremove(qp);
 #ifdef	DATUMREFCNT
 	free_nsp(nsp);
@@ -1271,6 +1341,7 @@ doupdate(msg, msglen, rrp, zone, savens, flags, cred)
 	u_char *cp1;
 	u_char data[BUFSIZ];
 	register HEADER *hp = (HEADER *)msg;
+	enum context context;
 #ifdef ALLOW_UPDATES
 	int zonenum;
 #endif
@@ -1288,6 +1359,11 @@ doupdate(msg, msglen, rrp, zone, savens, flags, cred)
 	GETSHORT(class, cp);
 	GETLONG(ttl, cp);
 	GETSHORT(dlen, cp);
+	if (!ns_nameok(dname, class, response_trans,
+		       ns_ownercontext(type, response_trans))) {
+		hp->rcode = FORMERR;
+		return (-1);
+	}
 	dprintf(3, (ddt, "doupdate: dname %s type %d class %d ttl %d\n",
 		    dname, type, class, ttl));
 	/*
@@ -1310,6 +1386,7 @@ doupdate(msg, msglen, rrp, zone, savens, flags, cred)
 	case T_X25:
 	case T_ISDN:
 	case T_NSAP:
+	case T_AAAA:
 	case T_LOC:
 #ifdef ALLOW_T_UNSPEC
 	case T_UNSPEC:
@@ -1331,20 +1408,36 @@ doupdate(msg, msglen, rrp, zone, savens, flags, cred)
 			hp->rcode = FORMERR;
 			return (-1);
 		}
+		if (!ns_nameok((char *)data, class, response_trans,
+			       (type == T_PTR)
+			       ? ns_ptrcontext(dname)
+			       : domain_ctx)) {
+			hp->rcode = FORMERR;
+			return (-1);
+		}
 		cp += n;
 		cp1 = data;
 		n = strlen((char *)data) + 1;
 		break;
 
-	case T_MINFO:
 	case T_SOA:
+		context = hostname_ctx;
+		goto soa_rp_minfo;
 	case T_RP:
+	case T_MINFO:
+		context = mailname_ctx;
+		/* FALLTHROUGH */
+	soa_rp_minfo:
 		n = dn_expand(msg, msg + msglen, cp,
 			      (char *)data, sizeof data);
 		if (n < 0) {
 			hp->rcode = FORMERR;
 			return (-1);
 		}
+		if (!ns_nameok((char *)data, class, response_trans, context)) {
+			hp->rcode = FORMERR;
+			return (-1);
+		}
 		cp += n;
 		cp1 = data + (n = strlen((char *)data) + 1);
 		n1 = sizeof(data) - n;
@@ -1355,6 +1448,14 @@ doupdate(msg, msglen, rrp, zone, savens, flags, cred)
 			hp->rcode = FORMERR;
 			return (-1);
 		}
+		if (type == T_RP)
+			context = domain_ctx;
+		else
+			context = mailname_ctx;
+		if (!ns_nameok((char *)cp1, class, response_trans, context)) {
+			hp->rcode = FORMERR;
+			return (-1);
+		}
 		cp += n;
 		cp1 += strlen((char *)cp1) + 1;
 		if (type == T_SOA) {
@@ -1381,6 +1482,11 @@ doupdate(msg, msglen, rrp, zone, savens, flags, cred)
 			hp->rcode = FORMERR;
 			return (-1);
 		}
+		if (!ns_nameok((char *)cp1, class, response_trans,
+			       hostname_ctx)) {
+			hp->rcode = FORMERR;
+			return (-1);
+		}
 		cp += n;
 
 		/* compute end of data */
@@ -1403,6 +1509,11 @@ doupdate(msg, msglen, rrp, zone, savens, flags, cred)
 			hp->rcode = FORMERR;
 			return (-1);
 		}
+		if (!ns_nameok((char *)cp1, class, response_trans,
+			       domain_ctx)) {
+			hp->rcode = FORMERR;
+			return (-1);
+		}
 		cp += n;
 		cp1 += (n = strlen((char *)cp1) + 1);
 		n1 = sizeof(data) - n;
@@ -1411,6 +1522,11 @@ doupdate(msg, msglen, rrp, zone, savens, flags, cred)
 			hp->rcode = FORMERR;
 			return (-1);
 		}
+		if (!ns_nameok((char *)cp1, class, response_trans,
+			       domain_ctx)) {
+			hp->rcode = FORMERR;
+			return (-1);
+		}
 		cp += n;
 		cp1 += strlen((char *)cp1) + 1;
 		n = cp1 - data;
@@ -1996,17 +2112,39 @@ sysquery(dname, class, type, nss, nsc, opcode)
 		}
 		qp->q_naddr = nsc;
 	} else {
+ fetch_a:
 		count = nslookup(nsp, qp, dname, "sysquery");
 		if (count <= 0) {
-			if (count < 0)
+			if (count < 0) {
 				syslog(LOG_INFO,
 				      "sysquery: nslookup reports danger (%s)",
 				       dname);
-			else
-				/* "." domain gets LOG_WARNING here. */
-				syslog(dname[0] ? LOG_INFO : LOG_WARNING,
-				       "sysquery: no addrs found for NS (%s)",
+				goto err2;
+			} else if (np && NAME(*np)[0] == '\0') {
+				syslog(LOG_WARNING,
+				   "sysquery: no addrs found for root NS (%s)",
 				       dname);
+				if (class == C_IN && !priming)
+					needs_prime_cache = 1;
+				goto err2;
+			}
+			if (np) {
+#ifdef DATUMREFCNT
+				free_nsp(nsp);
+				nsp[0] = NULL;
+#endif
+				np = np_parent(np);
+				n = findns(&np, class, nsp, &count, 0);
+				switch (n) {
+				case NXDOMAIN: /*FALLTHROUGH*/
+				case SERVFAIL:
+					syslog(LOG_DEBUG,
+					  "sysquery: findns error (%d) on %s?",
+					       n, dname);
+					goto err2;
+				}
+				goto fetch_a;
+			}
 			goto err2;
 		}
 	}
@@ -2020,7 +2158,7 @@ sysquery(dname, class, type, nss, nsc, opcode)
 		    "sysquery: send -> %s dfd=%d nsid=%d id=%d retry=%ld\n",
 		    sin_ntoa(nsa), qp->q_dfd, 
 		    ntohs(qp->q_nsid), ntohs(qp->q_id),
-		    qp->q_time));
+		    (long)qp->q_time));
 #ifdef DEBUG
 	if (debug >= 10)
 		fp_nquery(qp->q_msg, qp->q_msglen, ddt);
@@ -2053,7 +2191,7 @@ check_root()
 
 	priming = 0;
 	for (np = hashtab->h_tab[0]; np != NULL; np = np->n_next)
-		if (np->n_dname[0] == '\0')
+		if (NAME(*np)[0] == '\0')
 			break;
 	if (np == NULL) {
 		syslog(LOG_NOTICE, "check_root: Can't find root!\n");
@@ -2104,7 +2242,7 @@ check_ns()
 
 	curtime = (u_int32_t) tt.tv_sec;
 	for (np = hashtab->h_tab[0]; np != NULL; np = np->n_next) {
-		if (np->n_dname[0] != 0)
+		if (NAME(*np)[0] != '\0')
 			continue;
 		for (dp = np->n_data; dp != NULL; dp = dp->d_next) {
 			if (dp->d_type != T_NS)
@@ -2132,7 +2270,7 @@ check_ns()
 				    (tdp->d_ttl < curtime)) {
 					dprintf(3, (ddt,
 						"check_ns: stale entry '%s'\n",
-						    tnp->n_dname));
+						    NAME(*tnp)));
 					/* Cache invalidate the address RR's */
 					delete_all(tnp, dp->d_class, T_A);
 					found_arr = 0;
@@ -2177,25 +2315,23 @@ findns(npp, class, nsp, countp, flag)
 	nsp[0] = NULL;
 #endif
 
-	if (priming && (np == NULL || np->n_dname[0] == '\0'))
+	if (priming && (np == NULL || NAME(*np)[0] == '\0'))
 		htp = fcachetab;
 	else
 		htp = hashtab;
 
  try_again:
-	if (htp == fcachetab)
+	if (htp == fcachetab && class == C_IN && !priming)
 		needs_prime_cache = 1;
-	while (np == NULL && htp != NULL) {
-		dprintf(3, (ddt, "findns: using %s\n",
-			    htp == hashtab ? "cache" : "hints"));
+	if (np == NULL) {
+		/* find the root */
 		for (np = htp->h_tab[0]; np != NULL; np = np->n_next)
-			if (np->n_dname[0] == '\0')
+			if (NAME(*np)[0] == '\0')
 				break;
-		htp = (htp == hashtab ? fcachetab : NULL);	/* Fallback */
 	}
 	while (np != NULL) {
 		dprintf(5, (ddt, "findns: np %#lx '%s'\n",
-			    (u_long)np, np->n_dname));
+			    (u_long)np, NAME(*np)));
 		/* Look first for SOA records. */
 #ifdef ADDAUTH
 		if (!flag)
@@ -2251,12 +2387,24 @@ findns(npp, class, nsp, countp, flag)
 #endif
 			    !(dp->d_flags & DB_F_HINT)) {
 				dprintf(1, (ddt, "findns: stale entry '%s'\n",
-					    np->n_dname));
+					    NAME(*np)));
+#ifdef DATUMREFCNT
+				/*
+				 * We may have already added NS databufs
+				 * and are going to throw them away. Fix
+				 * fix reference counts. We don't need
+				 * free() them here as we just got them
+				 * from the cache.
+				 */
+				while (nspp > &nsp[0]) 
+					(*--nspp)->d_rcnt--;
+#endif
 				/* Cache invalidate the NS RR's. */
 #ifndef DATUMREFCNT
 				if (dp->d_ttl < tt.tv_sec)
 #endif
 					delete_all(np, class, T_NS);
+				nsp[0] = NULL;
 				goto try_parent;
 			}
 			if (nspp < &nsp[NSMAX-1]) {
@@ -2270,16 +2418,18 @@ findns(npp, class, nsp, countp, flag)
 		*countp = nspp - nsp;
 		if (*countp > 0) {
 			dprintf(3, (ddt, "findns: %d NS's added for '%s'\n",
-				    *countp, np->n_dname));
+				    *countp, NAME(*np)));
 			*nspp = NULL;
 			*npp = np;
 			return (OK);	/* Success, got some NS's */
 		}
 try_parent:
-		np = np->n_parent;
+		np = np_parent(np);
 	}
-	if (htp)
+	if (htp == hashtab) {
+		htp = fcachetab;
 		goto try_again;
+	}
 	dprintf(1, (ddt, "findns: No root nameservers for class %s?\n",
 		    p_class(class)));
 	if ((unsigned)class < MAXCLASS && norootlogged[class] == 0) {
@@ -2360,7 +2510,7 @@ finddata(np, class, type, hp, dnamep, lenp, countp)
 			 */
 			dprintf(3, (ddt,
 				    "finddata: stale entry '%s'\n",
-				    np->n_dname));
+				    NAME(*np)));
 			if (dp->d_zone == 0)
 				foundstale++;
 			continue;
@@ -2578,7 +2728,7 @@ delete_all(np, class, type)
 	register struct databuf *dp, *pdp;
 
 	dprintf(3, (ddt, "delete_all(%#lx:\"%s\" %s %s)\n",
-		    (u_long)np, np->n_dname, p_class(class), p_type(type)));
+		    (u_long)np, NAME(*np), p_class(class), p_type(type)));
 	pdp = NULL;
 	dp = np->n_data;
 	while (dp != NULL) {
