@@ -7,6 +7,7 @@
  * response to a query.
  */
 
+#include <sys/types.h>
 #include <sys/param.h>
 #include <sys/socket.h>
 #include <sys/file.h>
@@ -176,7 +177,7 @@ validate(dname, qdomain, server, type, class, data, dlen
 				return (VALID_NO_CACHE);
 		}
 #endif
-		if (!strcasecmp(dname, np->n_dname)) {
+		if (!strcasecmp(dname, NAME(*np))) {
       
 			/* if the name we seek is the same as that we have ns
 			 * records for, compare the data we have to see if it
