@@ -61,7 +61,7 @@ char copyright[] =
 
 #ifndef lint
 static char sccsid[] = "@(#)main.c	5.42 (Berkeley) 3/3/91";
-static char rcsid[] = "$Id: main.c,v 8.2 1995/12/22 10:20:42 vixie Exp $";
+static char rcsid[] = "$Id: main.c,v 8.3 1996/06/02 08:20:41 vixie Exp $";
 #endif /* not lint */
 
 /*
@@ -516,7 +516,14 @@ SetDefaultServer(string, local)
 			newServer, newDefPtr, 1);
     }
 
-    if (result == SUCCESS || result == NONAUTH) {
+    /* If we ask for an A record and get none back, but get an NS
+       record for the NS server, this is the NONAUTH case.
+       We must check whether we got an IP address for the NS
+       server or not.  */
+    if ((result == SUCCESS || result == NONAUTH) &&
+	((newDefPtr->addrList && newDefPtr->addrList[0] != 0) || 
+	 (newDefPtr->servers && newDefPtr->servers[0] &&
+			newDefPtr->servers[0]->addrList[0] != 0))) {
 	    /*
 	     *  Found info about the new server. Free the resources for
 	     *  the old server.
